/*
  adapter_pci_legacy_ethernet - Get PCI (and other) info for ethernet adapters.
  
  This file is part of the Linux lsvpd package.
  
  (C) Copyright IBM Corp. 2004, 2005, 2006
  
  Maintained by  Martin Schwenke <martins@au.ibm.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
 
  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
  $Id: adapter_pci_legacy_ethernet.c,v 1.13 2006/01/08 22:50:00 martins Exp $
*/
  
#define _GNU_SOURCE

#include <stdio.h>
#include <unistd.h>

typedef __uint64_t u64;
typedef __uint32_t u32;
typedef __uint16_t u16;
typedef __uint8_t  u8;

#include <net/if.h>
#include <linux/sockios.h>
#include <linux/ethtool.h>
#include <net/if_arp.h>

#include <sys/ioctl.h>
#include <stdlib.h>
#include <string.h>

#include <linux/pci_ids.h>

#include "util.h"
#include "adapter_pci_legacy_ethernet.h"
#include "adapter_pci_legacy.h"

static bool
pci_ethernet_line_scanner(const char * line,
			  void * x)
{
	char name[1024];
	char * s;
	int fd;
	struct ifreq ifr;
        struct ethtool_drvinfo drvinfo;
	unsigned int dom;
	unsigned char bus;
	unsigned char dev;
	unsigned char fun;
	adapter_pci_info_t ** l;
	adapter_pci_info_t * t;

	l = x;
	t = NULL;

	memset(name, 0, 1024);
	if (1 != sscanf(line, " %1023s", name))
		return true;

	if ((NULL == (s = memchr(name, ':', 1023))) || (s == name))
		return true;
	
	*s = '\0';
	
	if (-1 == (fd = socket(AF_INET, SOCK_DGRAM, 0)))
		return true;
	
	memset(&ifr, 0, sizeof(ifr));
	strncpy(ifr.ifr_name, name, IFNAMSIZ);
	
	if ((-1 == ioctl(fd, SIOCGIFHWADDR, &ifr)) ||
	    (ARPHRD_ETHER != ifr.ifr_hwaddr.sa_family))
		goto done;
	
	t = calloc(1, sizeof(adapter_pci_info_t));
	if (NULL == t)
		goto done;
	
	memcpy(t->eth_hwaddr, ifr.ifr_hwaddr.sa_data, PCI_ETHERNET_HWADDR_LEN);
	
	memset(&drvinfo, 0, sizeof(struct ethtool_drvinfo));
	drvinfo.cmd = ETHTOOL_GDRVINFO;
	ifr.ifr_data = (caddr_t)&drvinfo;
	if (-1 == ioctl(fd, SIOCETHTOOL, &ifr))
		goto bail;
	
	if (4 == sscanf(drvinfo.bus_info,
			"%04x:%02hhx:%02hhx.%hhx",
			&dom, &bus, &dev, &fun))
		t->dom = dom;
	else if (3 == sscanf(drvinfo.bus_info,
			     "%02hhx:%02hhx.%hhx", 
			     &bus, &dev, &fun))
		t->dom = ADAPTER_PCI_NODOM;
	else
		goto bail;
	
	t->bus = bus;
	t->dev = dev;
	t->fun = fun;
	
	memcpy(t->eth_fw_version, drvinfo.fw_version, 32);
	
	if (-1 == ioctl(fd, SIOCGIFMAP, &ifr))
		goto bail;
	
	t->irq       = (uint16_t) ifr.ifr_map.irq;

	t->eth_mem_start = (uint64_t) ifr.ifr_map.mem_start;
	t->eth_mem_end   = (uint64_t) ifr.ifr_map.mem_end;
	
	t->name = strndup(ifr.ifr_name, IFNAMSIZ);
	if (NULL == t->name)
		goto bail;

	t->class = PCI_CLASS_NETWORK_ETHERNET;

	t->scsi_driver = NULL;
	t->ide_channel = 0;

	t->next = *l;
	*l = t;

done:
	close(fd);
	return true;
bail:
	free(t);
	goto done;
}

/* Extra level of functions to allow testing against fixed files. */
static adapter_pci_info_t *
pci_ethernet_file(const char * filename)
{
	adapter_pci_info_t * l = NULL;
	line_scanner_harness(filename, pci_ethernet_line_scanner, &l);
	return l;
}


adapter_pci_info_t *
adapter_pci_legacy_ethernet(void)
{
	return pci_ethernet_file("/proc/net/dev");
}
