/*
  adapter_pci_legacy_ide - Get PCI (and other) information for ide adapters.
  
  This file is part of the Linux lsvpd package.
  
  (C) Copyright IBM Corp. 2004, 2005, 2006
  
  Maintained by  Martin Schwenke <martins@au.ibm.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
 
  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
  $Id: adapter_pci_legacy_ide.c,v 1.4 2006/01/16 05:31:00 martins Exp $
*/
  
#define _GNU_SOURCE

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <dirent.h>
#include <unistd.h>
#include <stdbool.h>
#include <stdlib.h>
#include <string.h>
#include <linux/pci_ids.h>

#include "util.h"
#include "adapter_pci_legacy.h"
#include "adapter_pci_legacy_ide.h"

#define ADAPTER_PCI_IDE_PROC_DIR "/proc/ide"

static void
pci_ide_parser (const char *          name,
		const unsigned char * data,
		adapter_pci_info_t ** ps)
{
	unsigned int  bus;
	unsigned int  devfun;
	unsigned int  channel;
	const char * t;

	adapter_pci_info_t * p = NULL;
	t = (const char *) data;

	if (t != (strstr(t, "pci bus")))
		return;
	if (3 != sscanf(t,
			"pci bus %x device %x %*s %*x %*s %*x channel %x",
			&bus,
			&devfun,
			&channel))
		return;

	p = calloc(1, sizeof(adapter_pci_info_t));

	if (NULL != p) {
		p->name = strdup(name);
		if (NULL == p->name)
			free(p);
		else {
			p->class = PCI_CLASS_STORAGE_IDE;

			p->dom = ADAPTER_PCI_NODOM;
			p->bus = (uint8_t) bus;
			p->dev = (uint8_t) devfun >> 3;
			p->fun = (uint8_t) devfun & 0x7;
			p->ide_channel = (uint8_t) channel;

			p->irq = 0;
			p->eth_mem_start = p->eth_mem_end = 0;
			p->scsi_driver = NULL;

			p->next = *ps;
			*ps = p;
		}
	}
}

/* Extra level of functions to allow testing against fixed files. */
static adapter_pci_info_t *
pci_ide_file(const char * proc_ide_dir)
{
	DIR * dir;
	struct dirent * e;
	unsigned int num;
	char * f = NULL;
	unsigned char * data = NULL;
	size_t len;
	
	adapter_pci_info_t * ret = NULL;

	dir = opendir(proc_ide_dir);

	if (NULL == dir)
		return NULL;

	while (NULL != (e = readdir(dir))) {
		if (0 != strncmp(e->d_name, "ide", 3))
			continue;
		if (1 != sscanf(&e->d_name[3], "%u", &num))
			continue;

		asprintf(&f, "%s/%s/config", proc_ide_dir, e->d_name);
		if (lsvpd_file_read(f, &data, &len)) {
			pci_ide_parser(e->d_name, data, &ret);
			free(data);
		}
		free(f);
	}

	closedir(dir);

	return ret;
}

adapter_pci_info_t *
adapter_pci_legacy_ide(void)
{
	return pci_ide_file(ADAPTER_PCI_IDE_PROC_DIR);
}
