/*
  adapter_pci_vpd_generic - Generic PCI VPD grabber.
  
  This file is part of the Linux lsvpd package.
  
  (C) Copyright IBM Corp. 2002, 2003, 2004, 2005, 2006
  
  Maintained by  Martin Schwenke <martins@au.ibm.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
 
  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
  $Id: adapter_pci_vpd_generic.c,v 1.5 2006/01/13 03:52:18 martins Exp $
*/
  
#include <stdlib.h>

#include "adapter_pci_vpd_generic.h"
#include "util.h"

#define RTT_END 0x0f

growbuf * adapter_pci_vpd_grab(struct adapter_pci_vpd_state * state)
{
	growbuf * ret = growbuf_new(1024);
	uint8_t * tmp;
	
	uint8_t tag, name;
	uint16_t data_len;

	do {
		/* Grab tag.*/
		if (1 != state->rf(state, &tag, 1))
			goto yikes;

		ret = growbuf_append(ret, &tag, 1);

		if (0 != (tag & (1 << 7))) {
			/* Large Resource Data Type tag, length follows. */
			if (2 != state->rf(state, (uint8_t *) &data_len, 2))
				goto yikes;
			ret = growbuf_append(ret, (uint8_t *) &data_len, 2);
			data_len = le16_to_cpu(data_len);
			
			name = tag & ~(1 << 7);
		} else {
			/* Small Resource Data Type tag, length embedded. */
			data_len = tag & 0x7;
			name = (tag >> 3) & 0x0f;
		}

		/* Now grab the data... if any. */
		if (0 != data_len) {
			tmp = malloc(data_len);
			if (state->rf(state, tmp, data_len) == data_len) {
				ret = growbuf_append(ret, tmp, data_len);
				free(tmp);
			} else {
				free(tmp);
				goto yikes;
			}
		}

	} while (RTT_END != name);

 bye:
	return ret;

 yikes:
	growbuf_free(ret);
	ret = NULL;
	goto bye;
}
