/*
  device-tree - lsvpd device-tree basics.
  
  This file is part of the Linux lsvpd package.
  
  (C) Copyright IBM Corp. 2006
  
  Maintained by  Martin Schwenke <martins@au.ibm.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
 
  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
  $Id: device_tree.c,v 1.2 2006/02/07 00:05:46 martins Exp $
*/

#include <stdlib.h>
#include <unistd.h>
#include <assert.h>

#include "lsvpd.h"
#include "util.h"
#include "init.h"

#include "device_tree.h"

#define LSVPD_DEVICE_TREE_SOURCE_DEFAULT "/proc/device-tree"

const char * lsvpd_dt_source_dir = NULL;
const char * lsvpd_dt_db_dir = NULL;

/********************************************************************/

static void
init(void)
{
	const char * d;
	char * f;

	d = getenv("LSVPD_DEVICE_TREE_DIR");

	if (NULL == d)
		d = LSVPD_DEVICE_TREE_SOURCE_DEFAULT;

	if (NULL != d) {
		f = lsvpd_relative_filename(d, "system-id");
		if (NULL != f) {
			if (0 == access(f, F_OK))
				lsvpd_dt_source_dir = d;
			free(f);
		}
	}

	if (NULL != lsvpd_dt_source_dir) {
		lsvpd_dt_db_dir = lsvpd_relative_filename(lsvpd_db_bus_dir,
							  "device-tree");
		assert(NULL != lsvpd_dt_db_dir);
		assert(lsvpd_mkdir(lsvpd_dt_db_dir));
	}
}

INIT(init);
