/*
  test_vpd - Test code for in memory and disk-based VPD utilities.
  
  This file is part of the Linux lsvpd package.
  
  (C) Copyright IBM Corp. 2004, 2005
  
  Maintained by  Martin Schwenke <martins@au.ibm.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
 
  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
  $Id: test_vpd.c,v 1.4 2005/05/26 05:43:10 martins Exp $
*/
  
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "vpd.h"
#include "growbuf.h"
#include "util.h"

/********************************************************************/

static void
die(const char * prog,
    const char * message,
    const char * sub_message)
{
	if (NULL != sub_message)
		fprintf(stderr, "%s: %s (%s)\n", prog, message, sub_message);
	else
		fprintf(stderr, "%s: %s\n", prog, message);

	exit(1);
}

int
main(int argc, char ** argv)
{
	growbuf * intext;
	growbuf * outtext;
	growbuf * t;
	vpd_list_t * vl;
	vpd_list_list_t * vll;

	FILE * f;

	char * data;
	size_t len;
	int err;

	if (argc != 6) {
		fprintf(stderr, "vpd intext outtext1 outtext2 dir sorttext\n");
		return 1;
	}

	if (!file_read_contents(argv[1], &data, &len, &err)) {
		die(argv[0], strerror(err), argv[1]);
	}

	intext = growbuf_append(growbuf_new(len), data, len);
	free(data);

	/* Test vpd_text_to_vpd_list and vpd_list_to_text. */
	if (NULL == (f = fopen(argv[2], "w"))) {
		die(argv[0], strerror(err), argv[2]);
	}

	if (NULL == (vl = vpd_text_to_vpd_list(intext))) {
		die(argv[0], "vpd_text_to_vpd_list failed", NULL);
	}

	outtext = growbuf_new(len);
	if (!vpd_list_to_text(vl, &outtext))
		die(argv[0], "vpd_list_to_text failed", NULL);

	fwrite(growbuf_data(outtext), growbuf_length(outtext), 1, f);
	vpd_list_free(vl);
	growbuf_free(outtext);
	fclose(f);

	/* Test vpd_text_to_vpd_list_list and vpd_list_list_to_text */

	vll = vpd_text_to_vpd_list_list(intext, "FC");

	if (NULL == vll)
		die(argv[0], "vpd_text_to_vpd_list_list failed", NULL);

	t = vpd_list_list_to_text(vll);
	if (NULL == t)
		die(argv[0], "vpd_list_list_to_text failed\n", NULL);

	if (NULL == (f = fopen(argv[3], "w"))) {
		die(argv[0], strerror(err), argv[3]);
	}
	fwrite(growbuf_data(t), growbuf_length(t), 1, f);
	fclose(f);
	growbuf_free(t);
	
	if (!vpd_list_list_to_node(vll, argv[4]))
		die(argv[0], "vpd_list_list_to_node failed\n", NULL);
	
	vpd_list_list_free(vll);

	/* test vpd_list_sort. */
	if (NULL == (f = fopen(argv[5], "w"))) {
		die(argv[0], strerror(err), argv[5]);
	}

	if (NULL == (vl = vpd_text_to_vpd_list(intext))) {
		die(argv[0], "vpd_text_to_vpd_list failed", NULL);
	}

	vpd_list_sort(&vl, (const char * []) { "FC", "YL", NULL } );

	outtext = growbuf_new(len);
	if (!vpd_list_to_text(vl, &outtext))
		die(argv[0], "vpd_list_to_text failed", NULL);

	fwrite(growbuf_data(outtext), growbuf_length(outtext), 1, f);
	vpd_list_free(vl);
	growbuf_free(outtext);
	fclose(f);

		
	growbuf_free(intext);

	return 0;
}
